/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Scotty is free software: you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later version.

Scotty is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
Scotty. If not, see <https://www.gnu.org/licenses/>.
*/

package lastfm

import (
	"net/url"

	"github.com/shkh/lastfm-go/lastfm"
	"github.com/spf13/viper"
	"go.uploadedlobster.com/scotty/internal/auth"
	"go.uploadedlobster.com/scotty/internal/models"
	"golang.org/x/oauth2"
)

type LastfmApiBackend struct {
	client *lastfm.Api
}

func (b *LastfmApiBackend) Name() string { return "lastfm" }

func (b *LastfmApiBackend) FromConfig(config *viper.Viper) models.Backend {
	clientId := config.GetString("client-id")
	clientSecret := config.GetString("client-secret")
	b.client = lastfm.New(clientId, clientSecret)
	return b
}

func (b *LastfmApiBackend) OAuth2Strategy(redirectUrl *url.URL) auth.OAuth2Strategy {
	return lastfmStrategy{
		client:      b.client,
		redirectUrl: redirectUrl,
	}
}

func (b *LastfmApiBackend) OAuth2Setup(token oauth2.TokenSource) error {
	t, err := token.Token()
	if err != nil {
		return err
	}
	return b.client.LoginWithToken(t.AccessToken)
}
